#!/usr/bin/env python
import json
import getpass
import getopt
import pds
import sys
import globals
import uuid
import requests

def print_usage():
    print('Usage:')
    print('list_algos.py [-s appkey:secret] [-e environment]')


def print_algos(token, onBehalfOf):
    algos = pds.get_algo_list(token, onBehalfOf)

    for i in algos["instruments"]:
        od = json.loads(i["od"])
        ui_rep = i["n"]
        if "ui_rep" in od:
            ui_rep = od["ui_rep"]

        algo_ver = "MISSING!"
        if "algo_version" in od:
            algo_ver = od["algo_version"]

        print("{0} ('{1}') => id: {2}, ver: {3}, algo_version: {4}".format(
            i["n"],
            ui_rep,
            i["i"],
            i["iv"],
            algo_ver
            ))

def main(argv):
    env = ""

    try:
        opts, args = getopt.getopt(argv, "s:e:", ["appkeysecret=", "env="])
    except getopt.GetoptError:
        print_usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print_usage()
            sys.exit()
        elif opt in ("-s", "--appkeysecret"):
            appkey = arg
        elif opt in ("-e", "--env"):
            env = arg

    theRequestId = "ttsdkalgo-listalgo--{0}".format(uuid.uuid4())
    globals.environment(env)
    token = globals.get_access_token(appkey, theRequestId)
    
    print_algos(token, theRequestId)


if __name__ == "__main__":
    main(sys.argv[1:])
