import json
import requests
import base64
import globals
import uuid

def upload_resource(desc, res_id, meta, access_token, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    url = globals.get_pds_url() + "api/1/resources/" + str(res_id) + "?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    headers = {"Authorization": "Bearer " + access_token}

    print("Uploading " + desc + " (resource " + str(res_id) + ") for " + onBehalfOf + " requestId=" + requestId)
    resp = requests.post(url, headers=headers, files=meta)
    resp.raise_for_status()


def delete_resource(res_id, access_token, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    url = globals.get_pds_url() + "api/1/resources/" + str(res_id) + "?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    headers = {"Authorization": "Bearer " + access_token}

    print("Deleting resource " + str(res_id) + " for " + onBehalfOf + " requestId=" + requestId)
    resp = requests.delete(url, headers=headers)
    print("   success? " + str(resp.ok))


def get_algo_list(access_token, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    url = globals.get_pds_url() + "api/1/instruments?marketIds=1005&slim=false&request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    headers = {"Authorization": "Bearer " + access_token}

    resp = requests.get(url, headers=headers)
    resp.raise_for_status()

    return json.loads(resp.content)


def share_algo(instrId, personId, email, access_token, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    data = ({"algosh": [
            {"id":instrId, "sharedPId": personId, "pRead": False,
              "pExecute": True, "sEmail": email, "sNotes": ""} ]})
    url = globals.get_algostore_url() + "api/1/algos/sharing?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    headers = {'Authorization': "Bearer " + access_token,
              'Content-Type': 'application/json'}

    print("sharing algo " + str(instrId) + " for " + onBehalfOf + " requestId=" + requestId)
    resp = requests.post(url, data=json.dumps(data), headers=headers)
    resp.raise_for_status()


def unshare_algo(instrId, personId, email, access_token, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    data = ({"algosh": [
            {"id":instrId, "sharedPId": personId} ]})
    url = globals.get_algostore_url() + "api/1/algos/unsharing?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    headers = {'Authorization': "Bearer " + access_token,
              'Content-Type': 'application/json'}
    print("unsharing algo " + str(instrId) + " for " + onBehalfOf + " requestId=" + requestId)
    resp = requests.post(url, data=json.dumps(data), headers=headers)
    resp.raise_for_status()
