#!/usr/bin/env python
import json
import getpass
import sys
import getopt
import requests
import pds
import globals
import list_algos
import uuid


def undeploy_algo(instr_id, access_token, onBehalfOf):
    headers = {"Authorization": "Bearer " + access_token}

    requestId = "ttsdkalgo-undeploy--{0}".format(uuid.uuid4())
    # Get the instrument definition first
    url = globals.get_pds_url() + "api/1/instruments/" + str(instr_id) + "?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    print("Requesting instrument for {0} with requestId={1}".format(onBehalfOf, requestId))
    resp = requests.get(url, headers=headers)

    if resp.status_code == 404:
        print("")
        print("Algo instrument not found. Here is a list of algos accessible to you:")
        print("")
        list_algos.print_algos(access_token, onBehalfOf)
        print("")
        return

    resp.raise_for_status()

    instr_def = json.loads(resp.content)["instrument"]

    # Delete the instrument
    requestId = "ttsdkalgo-undeploy--{0}".format(uuid.uuid4())
    print("Deleting instrument " + str(instr_id) + " for " + onBehalfOf + " request_id=" + requestId)
    url = globals.get_pds_url() + "api/1/algoinstruments/" + str(instr_id) + "?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    resp = requests.delete(url, headers=headers)
    resp.raise_for_status()

    if "od" in instr_def:
        od = json.loads(instr_def["od"])

        if "meta_res_id" in od:
            res_id = str(od["meta_res_id"])
            pds.delete_resource(res_id, access_token, onBehalfOf)

        if "algo_res_id" in od:
            res_id = str(od["algo_res_id"])
            pds.delete_resource(res_id, access_token, onBehalfOf)

    return


def print_usage():
    print("")
    print('Usage:')
    print('undeploy.py -a algo_name_or_instrument_id -e target_TTW_environment -s appkey:secret')


def main(argv):
    user = ""
    password = ""
    algo = ""
    env = ""
    instr_id = "0"

    try:
        opts, args = getopt.getopt(argv, "h:a:s:e:", ["algo=", "appkeysecret=", "env="])
    except getopt.GetoptError:
        print_usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print_usage()
            sys.exit()
        elif opt in ("-a", "--algo"):
            algo = arg
        elif opt in ("-s", "--appkeysecret"):
            appkey = arg
        elif opt in ("-e", "--env"):
            env = arg

    if algo.strip() == "":
        print_usage()
        sys.exit(2)

    if appkey.strip() == "":
        print_usage()
        sys.exit(2)

    if env.strip() == "":
        env = getpass.getpass("environment: ")

    globals.environment(env)
    theRequestId = "ttsdkalgo-undeploy--{0}".format(uuid.uuid4())

    print("Obtaining access token")
    token = globals.get_access_token(appkey, theRequestId)

    print("Looking for '{0}'".format(algo))
    algos = pds.get_algo_list(token, theRequestId)
    for al in algos["instruments"]:
        if str(al["i"]) == algo or al["n"] == algo:
            instr_id = al["i"]
            alias = al["n"]
            if "ui_rep" in al:
                alias = al["ui_rep"]
            print("Found. {0} ('{1}') => id: {2}, ver: {3}".format(al["n"], alias, al["i"], al["iv"]))
            break

    undeploy_algo(instr_id, token, theRequestId)


    print("")
    print("Successfully undeployed sdk algo: InstrumentId: {0}".format(instr_id))
    print("")

if __name__ == "__main__":
    main(sys.argv[1:])
