#!/usr/bin/env python
import json
import getpass
import sys
import getopt
import uuid

import requests
import pds
import globals
import list_algos




def print_usage():
    print("")
    print('Usage:')
    print('unshare.py -a algo_name_or_instrument_id -e target_TTW_environment -s appkey:secret -m share_with_email')


def main(argv):
    algo = ""
    env = ""
    shareEmail = ""
    instr_id = "0"

    try:
        opts, args = getopt.getopt(argv, "h:a:s:e:m:", ["algo=", "appkeysecret=", "env=", "email="])
    except getopt.GetoptError:
        print_usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print_usage()
            sys.exit()
        elif opt in ("-a", "--algo"):
            algo = arg
        elif opt in ("-s", "--appkeysecret"):
            appkey = arg
        elif opt in ("-e", "--env"):
            env = arg
        elif opt in ("-m", "--email"):
            shareEmail = arg

    if algo.strip() == "":
        print_usage()
        sys.exit(2)

    if env.strip() == "":
        env = getpass.getpass("environment: ")

    globals.environment(env)
    theRequestId = "ttsdkalgo-deploy--{0}".format(uuid.uuid4())

    print("Obtaining access token")
    token = globals.get_access_token(appkey, theRequestId)
    userId = globals.get_user_id(token, theRequestId)

    print("Looking for '{0}'".format(algo))
    algos = pds.get_algo_list(token, theRequestId)
    found = False
    for al in algos["instruments"]:
        if str(al["i"]) == algo or al["n"] == algo:
            instr_id = al["i"]
            alias = al["n"]
            uui = al["uui"]
            if userId == uui:
                found = True
            if "ui_rep" in al:
                alias = al["ui_rep"]
            print("Found. {0} ('{1}') => id: {2}, ver: {3}".format(al["n"], alias, al["i"], al["iv"]))
            break

    if not found:
        print("")
        print("Algo instrument not found. Here is a list of algos accessible to you:")
        print("")
        list_algos.print_algos(token, theRequestId)
        print("")
    else:
        personId = globals.get_person_id(shareEmail, token, theRequestId)
        pds.unshare_algo(instr_id, personId, shareEmail, token, theRequestId)
        print("")
        print("Successfully unshared sdk algo: InstrumentId: {0} with {1}".format(instr_id, shareEmail))
        print("")

if __name__ == "__main__":
    main(sys.argv[1:])
